(function() {
  var FieldContentParser, FieldFinder, FieldNames, FieldNamesParser, FieldParser, FieldRegexpFactory, MandatoryFieldDetector, PatternNameInjector, XRegExp, parsePattern;

  XRegExp = require('xregexp').XRegExp;

  parsePattern = require('./FieldPeg').parse;

  FieldParser = class FieldParser {
    constructor(fieldPatterns) {
      this.fieldPatterns = fieldPatterns;
      this.fieldParsers = {};
      this.regexpFactory = new FieldRegexpFactory();
    }

    parse(fieldHeader, fieldContent) {
      var fieldMetadata, parser, regexpSt, result;
      if (this.fieldParsers[fieldHeader] == null) {
        fieldMetadata = this.fieldPatterns[fieldHeader];
        if (!(fieldMetadata != null)) {
          throw new Error(`Metadata not found for field ${fieldHeader}.`);
        }
        if (fieldHeader === "77E") {
          throw new Error("Parsing of field 77E is not supported."); //this field has a very strange pattern and multiple fields with the same name
        }
        regexpSt = this.regexpFactory.createRegexp(fieldMetadata.pattern, fieldMetadata.fieldNames);
        this.fieldParsers[fieldHeader] = new FieldContentParser(regexpSt, new FieldNames(fieldMetadata.fieldNames));
      }
      parser = this.fieldParsers[fieldHeader];
      result = parser.parse(fieldContent);
      return result;
    }

  };

  FieldContentParser = class FieldContentParser {
    constructor(regexpSt1, fieldNames1) {
      this.regexpSt = regexpSt1;
      this.fieldNames = fieldNames1;
      this.regexp = new XRegExp(this.regexpSt);
    }

    parse(fieldValue) {
      var fieldName, j, len, match, ref, result;
      match = this.regexp.xexec(fieldValue);
      if (match == null) {
        throw new Error(`Unable to parse '${fieldValue}' with regexp '${this.regexpSt}'.`);
      }
      result = {};
      ref = this.fieldNames.flatNames;
      for (j = 0, len = ref.length; j < len; j++) {
        fieldName = ref[j];
        if ((match[fieldName] != null)) {
          result[FieldNamesParser.unescape(fieldName)] = match[fieldName];
        }
      }
      return result;
    }

  };

  FieldRegexpFactory = class FieldRegexpFactory {
    createRegexp(pattern, fieldNamesString) {
      var fieldNames, fieldNamesSection, head, i, j, k, leftMandatory, len, len1, mandatoryFieldDetector, patternPart, patternParts, regexpPart, regexps, result, rightMandatory;
      patternParts = pattern.split('$');
      fieldNames = new FieldNames(fieldNamesString);
      if (patternParts.length !== fieldNames.names.length) {
        throw new Error('Different count of lines in pattern and field names.');
      }
      regexps = [];
      for (i = j = 0, len = patternParts.length; j < len; i = ++j) {
        patternPart = patternParts[i];
        fieldNamesSection = fieldNames.names[i];
        regexps.push(this._createRegexpCore(patternPart, fieldNamesSection));
      }
      mandatoryFieldDetector = new MandatoryFieldDetector();
      [head, ...regexps] = regexps;
      result = head.regexp;
      leftMandatory = mandatoryFieldDetector.containsMandatory(head.tree);
      for (k = 0, len1 = regexps.length; k < len1; k++) {
        regexpPart = regexps[k];
        rightMandatory = mandatoryFieldDetector.containsMandatory(regexpPart.tree);
        if (leftMandatory && rightMandatory) {
          result = result + "\r\n" + regexpPart.regexp; //not 100% correct -- the newlines should be parts of the sequences -- [a]$b --> (a\r\n)?b instead of (a)?(\r\n)?b
        } else {
          result = result + `(\r\n)?${regexpPart.regexp}`;
        }
      }
      result = "^" + result + "$";
      return result;
    }

    _createRegexpCore(pattern, fieldNames) {
      var injector, parsedPattern, prefix, regexp;
      if (pattern[0] === ':') { //make the leading colon optional not to enforce it in the field value
        prefix = ":?";
        pattern = pattern.substring(1);
      }
      parsedPattern = parsePattern(pattern);
      injector = new PatternNameInjector();
      injector.injectNames(fieldNames, parsedPattern);
      regexp = this._visitNode(parsedPattern);
      if ((prefix != null)) {
        regexp = prefix + regexp;
      }
      return {
        tree: parsedPattern,
        regexp: regexp
      };
    }

    _visitNodes(array) {
      var node, result;
      result = ((function() {
        var j, len, results;
        results = [];
        for (j = 0, len = array.length; j < len; j++) {
          node = array[j];
          results.push(this._visitNode(node));
        }
        return results;
      }).call(this)).join("");
      return result;
    }

    _visitNode(node) {
      var rxName, rxOptional, value;
      switch (node.type) {
        case 'literal':
          return this._visitLiteral(node);
        case 'sequence':
          rxOptional = node.optional ? "?" : "";
          rxName = (node.name != null) ? `?<${node.name}>` : "";
          value = this._visitNodes(node.parts);
          if (!node.optional && (node.name == null)) { //no need to add parentheses
            return value;
          }
          if (node.parts.length === 1 && (node.name == null)) {
            if (node.optional && /^\(.*\)$/.test(value)) {
              return value + "?"; //there are already parentheses
            }
          }
          return `(${rxName}${value})${rxOptional}`;
        case 'field':
          return this._visitField(node);
        default:
          throw new Error(`Unknown node type ${node.type}: ` + node);
      }
    }

    _visitField(field) {
      var count, exact, lines, ref, ref1, result, rxCount, rxLines, rxName, rxSet, set;
      count = field.count;
      set = field.set;
      lines = (ref = field.lines) != null ? ref : 1;
      exact = (ref1 = field.exact) != null ? ref1 : false;
      rxSet = (function() {
        switch (set) {
          case 'e':
            return ' ';
          case 'z':
            return '[\\s\\S]';
          default:
            return '.';
        }
      })();
      rxCount = exact ? `{${count}}` : `{1,${count}}`;
      rxName = (field.name != null) ? `?<${field.name}>` : "";
      rxLines = lines > 1 ? `(\r\n${rxSet}${rxCount}){0,${lines - 1}}` : "";
      if (rxName !== "") {
        result = `(${rxName}${rxSet}${rxCount}${rxLines})`;
      } else {
        result = `${rxSet}${rxCount}${rxLines}`;
      }
      return result;
    }

    _visitLiteral(node) {
      if ((node.name != null)) {
        return `(?<${node.name}>${node.value})`;
      }
      return node.value;
    }

  };

  FieldNames = class FieldNames {
    constructor(fieldNamesString1) {
      var fieldNamesPart, fieldNamesParts, j, k, l, len, len1, len2, name, names, ref, section;
      this.fieldNamesString = fieldNamesString1;
      fieldNamesParts = this.fieldNamesString.split('$');
      this.names = [];
      for (j = 0, len = fieldNamesParts.length; j < len; j++) {
        fieldNamesPart = fieldNamesParts[j];
        if (fieldNamesPart === "") { //special handling of empty list
          fieldNamesPart = "(Value)";
        }
        names = FieldNamesParser.parseFieldNames(fieldNamesPart);
        this.names.push(names);
      }
      this.flatNames = [];
      ref = this.names;
      for (k = 0, len1 = ref.length; k < len1; k++) {
        section = ref[k];
        for (l = 0, len2 = section.length; l < len2; l++) {
          name = section[l];
          this.flatNames.push(name);
        }
      }
    }

  };

  FieldNamesParser = (function() {
    class FieldNamesParser {
      static parseFieldNames(fieldNamesString) {
        var names;
        if (fieldNamesString === "") {
          return [];
        }
        names = [];
        this._fieldNamesRegExp.forEach(fieldNamesString, (match) => {
          var escaped;
          escaped = this.escape(match[1]);
          return names.push(escaped);
        });
        if (names.length === 0) {
          throw new Error("Strange field names: " + fieldNamesString);
        }
        return names;
      }

      static escape(name) {
        return name.replace(new XRegExp(" ", "g"), "_");
      }

      static unescape(name) {
        return name.replace(new XRegExp("_", "g"), " ");
      }

    };

    FieldNamesParser._fieldNamesRegExp = new XRegExp("\\((.*?)\\)");

    return FieldNamesParser;

  }).call(this);

  PatternNameInjector = class PatternNameInjector {
    injectNames(names, parsedPattern) {
      var result;
      this.remainingNames = names;
      this.pattern = parsedPattern;
      result = this._visitNode(parsedPattern);
      if (this.remainingNames.length > 0) {
        throw new Error("Remaining names after name injection: " + this.remainingNames.toString());
      }
      return result;
    }

    _visitNode(node) {
      var child, j, len, ref;
      switch (node.type) {
        case 'literal':
          return this._visitLiteral(node);
        case 'sequence':
          ref = node.parts;
          for (j = 0, len = ref.length; j < len; j++) {
            child = ref[j];
            this._visitNode(child);
          }
          return node;
        case 'field':
          return this._visitField(node);
        default:
          throw new Error(`Unknown node type ${node.type}: ` + node);
      }
    }

    _visitLiteral(node) {
      var name;
      if (node.value === 'N' && (this.remainingNames[0] != null) && /(_|\b)sign(_|\b)/i.test(this.remainingNames[0])) {
        [name, ...this.remainingNames] = this.remainingNames;
        node.name = name;
      }
      return node;
    }

    _visitField(node) {
      if (node.set === 'e') { //space doesnt' get name
        return node;
      }
      this._attachNameToField(node);
      return node;
    }

    _attachNameToField(node) {
      var commonAncestor, currentFieldPath, i, left, length, name, newNode, righmostFieldPath, right;
      if (this.remainingNames.length === 0) {
        return;
      }
      if (this.remainingNames.length === 1) {
        righmostFieldPath = new FieldFinder(function(field) {
          return true;
        }).findPath(this.pattern);
        currentFieldPath = new FieldFinder(function(field) {
          return field === node;
        }).findPath(this.pattern);
        length = Math.min(righmostFieldPath.length, currentFieldPath.length);
        i = 0;
        commonAncestor = null;
        while (i < length && righmostFieldPath[i] === currentFieldPath[i]) {
          commonAncestor = righmostFieldPath[i];
          i = i + 1;
        }
        if (i < length) {
          //rewrite the pattern tree to name the remaining fields as a sequence
          if (commonAncestor.type !== 'sequence') {
            throw new Error('Common ancestor should be a sequence: ' + JSON.stringify(commonAncestor));
          }
          left = commonAncestor.parts.indexOf(currentFieldPath[i]);
          right = commonAncestor.parts.indexOf(righmostFieldPath[i]);
          if (left === -1 || right === -1) {
            throw new Error(`Left: ${left} Right: ${right}`);
          }
          newNode = {
            type: 'sequence',
            optional: false,
            parts: commonAncestor.parts.slice(left, +right + 1 || 9e9)
          };
          commonAncestor.parts = commonAncestor.parts.slice(0, left).concat([newNode]).concat(commonAncestor.parts.slice(right + 1));
          node = newNode;
        }
      }
      [name, ...this.remainingNames] = this.remainingNames;
      return node.name = name;
    }

  };

  // Depth-first, right-to-left traverser
  FieldFinder = class FieldFinder {
    constructor(predicate) {
      this.predicate = predicate;
    }

    findPath(tree) {
      var path;
      path = [];
      this._visitNode(tree, path);
      return path;
    }

    _visitNode(node, path) {
      var child, j, ref;
      switch (node.type) {
        case 'literal':
          return false;
        case 'field':
          if (this.predicate(node)) {
            path.push(node);
            return true;
          } else {
            return false;
          }
          break;
        case 'sequence':
          path.push(node);
          ref = node.parts;
          for (j = ref.length - 1; j >= 0; j += -1) {
            child = ref[j];
            if (this._visitNode(child, path)) {
              return true;
            }
          }
          path.pop();
          return false;
        default:
          throw new Error(`Unknown node type ${node.type}: ` + node);
      }
    }

  };

  MandatoryFieldDetector = class MandatoryFieldDetector {
    constructor() {}

    containsMandatory(tree) {
      return this._visitNode(tree);
    }

    _visitNode(node) {
      var child, j, len, ref;
      switch (node.type) {
        case 'literal':
          return false;
        case 'sequence':
          if (!node.optional) {
            ref = node.parts;
            for (j = 0, len = ref.length; j < len; j++) {
              child = ref[j];
              if (this._visitNode(child)) {
                return true;
              }
            }
          }
          return false;
        case 'field':
          return true;
        default:
          throw new Error(`Unknown node type ${node.type}: ` + node);
      }
    }

  };

  "n  - [0-9] -   Digits\nd   - [0-9]+,[0-9]* problem with total length   -   Digits with decimal comma\na   - [A-Z]  -   Uppercase letters\nc   - [0-9A-Z] -   Uppercase alphanumeric\ne   - [ ]   -   Space\nx   - [0-9a-zA-Z/\-\?:\(\)\.,&apos;\+ ]   -   SWIFT character set\nz   -  [0-9a-zA-Z!&quot;%&amp;\*;&lt;&gt; \.,\(\)/=&apos;\+:\?@#&#x0d;&#x0a;\{\-_]  -   SWIFT extended character set\n//h      -   Uppercase hexadecimal\n//y      -   Upper case level A ISO 9735 characters\n\n\nspecials:\nISIN\nN\n//\n,\n/\n\nnew line:\n$\n";

  module.exports.FieldRegexpFactory = FieldRegexpFactory;

  module.exports.FieldFinder = FieldFinder;

  module.exports.FieldNamesParser = FieldNamesParser;

  module.exports.FieldContentParser = FieldContentParser;

  module.exports.FieldParser = FieldParser;

  module.exports.FieldNames = FieldNames;

}).call(this);

//# sourceMappingURL=FieldRegexpFactory.js.map
